import subprocess

from dataclasses import dataclass
from pathlib import Path
from string import Template
from typing import List


from click import echo, secho

from tzar import config


@dataclass
class CLIArguments:
    filename: str
    directory: str = ""
    verbose: bool = False
    forced_extension: str = ""


@dataclass
class CLITemplate:
    extensions: List[str]
    compress: str = ""
    extract: str = ""
    show: str = ""
    verbose: str = "v"

    def matches_filename(self, file_path: Path, forced_extension: str = "") -> bool:
        suffix = forced_extension or "".join(file_path.suffixes)
        if suffix == "":
            raise ValueError(
                "File contains no extension. You can force to use one by adding the `--extension=` or `-e` parameter"
            )
        return suffix in self.extensions

    def build_command(
        self, command_template, filename: str, directory: str, verbose: bool
    ) -> str:
        verbose_arg = self.verbose if verbose else ""
        template = Template(command_template)
        command = template.substitute(verbose=verbose_arg, directory=directory, filename=filename)
        echo(f"Running: {command}")
        return command

    def run_compress(self, args):
        command = self.build_command(
            self.compress, args.filename, args.directory, args.verbose
        )
        return subprocess.run(command, shell=True).returncode == 0

    def run_extract(self, args):
        command = self.build_command(
            self.extract, args.filename, args.directory, args.verbose
        )
        return subprocess.run(command, shell=True).returncode == 0

    def run_show(self, args):
        command = self.build_command(
            self.show, args.filename, args.directory, args.verbose
        )
        return subprocess.run(command, shell=True).returncode == 0


@dataclass
class CLITemplateCollection:
    cli_templates: List[CLITemplate]

    @classmethod
    def from_config(cls: "CLITemplateCollection") -> "CLITemplateCollection":
        configs = config.read()
        return cls(
            cli_templates=[CLITemplate(**fields) for c in configs for fields in c.values()]
        )

    def get_templates(self, file_path: Path, forced_extension: str = ""):
        return [
            template
            for template in self.cli_templates
            if template.matches_filename(file_path, forced_extension=forced_extension)
        ]

    def compress(self, args: CLIArguments):
        templates = self.get_templates(
            Path(args.filename), forced_extension=args.forced_extension
        )
        for template in templates:
            if template.run_compress(args):
                secho("Archive compressed successfully!", fg="green")
                return
        if len(templates) == 0:
            secho("No command found for that file extension.", err=True, fg="red")
        else:
            secho("All attempts failed!", err=True, fg="red")

    def extract(self, args: CLIArguments):
        templates = self.get_templates(
            Path(args.filename), forced_extension=args.forced_extension
        )
        for template in templates:
            if template.run_extract(args):
                secho("Archive extracted successfully!", fg="green")
                return
        if len(templates) == 0:
            secho("No command found for that file extension.", err=True, fg="red")
        else:
            secho("All attempts failed!", err=True, fg="red")

    def show(self, args: CLIArguments):
        templates = self.get_templates(
            Path(args.filename), forced_extension=args.forced_extension
        )
        for template in templates:
            if template.run_show(args):
                return
        if len(templates) == 0:
            secho("No command found for that file extension.", err=True, fg="red")
        else:
            secho("All attempts failed!", err=True, fg="red")



import os
from pathlib import Path
from typing import List
from urllib import request

import toml
from click import confirm, get_app_dir, secho

DEFAULT_CONFIG_URL = (
    "https://raw.githubusercontent.com/DanielVZ96/tzar/main/config/default.toml"
)


def read() -> List[dict]:
    if "TZAR_CONFIG" in os.environ:
        config_path = os.environ["TZAR_CONFIG"]
    else:
        config_path = get_app_dir("tzar")
    os.makedirs(config_path, exist_ok=True)
    config_path = Path(config_path)
    configs = [dict(toml.load(config)) for config in config_path.glob("*.toml")]
    if configs:
        return configs
    elif prompt_config_download(config_path):
        return read()

    secho("No config files found!", fg="red", bold=True, err=True)
    return []


def prompt_config_download(config_path: Path) -> bool:
    secho(
        f"No config files where found. You can download the default one from: {DEFAULT_CONFIG_URL}",
        fg="cyan",
    )
    if confirm(f"Download default config file?"):
        filename, _ = request.urlretrieve(
            DEFAULT_CONFIG_URL, config_path / "default.toml"
        )
        secho(f"Config downloaded to: {filename}", fg="green")
        return True
    return False



import click

from tzar.templates import CLITemplateCollection, CLIArguments


@click.group()
def cli():
    pass


@cli.command()
@click.argument("source")
@click.argument("destination")
@click.option("--verbose", "-v", is_flag=True, help="Print more output.")
@click.option(
    "--extension", "-e", help="Force the command to use a certain file extension."
)
def extract(source, destination, verbose, extension):
    CLITemplateCollection.from_config().extract(
        CLIArguments(filename=source, directory=destination, verbose=verbose)
    )


@cli.command()
@click.argument("source")
@click.argument("destination")
@click.option("--verbose", "-v", is_flag=True, help="Print more output.")
@click.option(
    "--extension", "-e", help="Force the command to use a certain file extension."
)
def compress(source, destination, verbose, extension):
    CLITemplateCollection.from_config().compress(
        CLIArguments(
            filename=destination,
            directory=source,
            verbose=verbose,
            forced_extension=extension,
        )
    )


@cli.command()
@click.argument("source")
@click.option("--verbose", "-v", is_flag=True, help="Print more output.")
@click.option(
    "--extension", "-e", help="Force the command to use a certain file extension."
)
def list(source, verbose, extension):
    CLITemplateCollection.from_config().show(
        CLIArguments(
            filename=source,
            verbose=verbose,
            forced_extension=extension,
        )
    )

def run():
    cli()

if __name__ == "__main__":
    run()
